unit AceImLib;

interface

{$ifdef WIN32}
  uses windows, sysutils, classes, aceout;
{$else}
  uses wintypes, winprocs, sysutils, classes, aceout;
{$endif}


  procedure AceImgLibFromStream(AceCanvas: TAceCanvas; Stream: TStream;
             FileExt: String; Rect: TRect);

implementation

uses TMultiP, acefile, aceutil, acememou, graphics, aceimg;

const
{
  AceIL_ImageLibType
  This constant is registered with a procedure that needs to be called
  when ACE reads this print type within the AceFile.
}
  AceIL_ImageLibType = 1010;


procedure AceImgLibFromStream(AceCanvas: TAceCanvas; Stream: TStream;
           FileExt: String; Rect: TRect);
var
  Size: LongInt;
  ReadStream: TMemoryStream;
begin
  if Stream <> nil then
  begin
    Size := Stream.Size;
    Stream.Position := 0;
    AceIsBlob(Stream);
    ReadStream := nil;
    try
      ReadStream := TMemoryStream.Create;
      ReadStream.CopyFrom(Stream, Size - Stream.Position);

      AceCanvas.AceOutput.AceFile.WriteType(AceIL_ImageLibType);
      AceCanvas.AceOutput.AceFile.WriteRect(Rect);
      AceCanvas.AceOutput.AceFile.WriteString(FileExt);
      AceCanvas.AceOutput.AceFile.WriteStream(ReadStream);

    finally
      if ReadStream <> nil then ReadStream.Free;
    end;
  end;
end;

procedure AcePrintImageLibImage(Handle: THandle; AceFile: TAceAceFile);
var
  R: TRect;
  Stream: TMemoryStream;
  TempName: String;
  Image: TPMultiImage;
  FStr: TFileStream;
  Ext: String;
begin
  Image := nil;
  FStr := nil;

  Stream := TMemoryStream.Create;
  try
    { Read the rectangle to print in }
    AceFile.ReadRect(R);
    { Scale the rectangle to the output coordinates }
    R := AceScaleRect(Handle, AceFile.PixelsPerInchX, AceFile.PixelsPerInchY,
                      R, AceFile.HorzScale, AceFile.VertScale);
    { Read file ext }
    AceFile.ReadString(Ext);
    { Read the image data }
    AceFile.ReadStream( Stream );

    { 'IL' is just a prefix for ImageLib }
    TempName := AceGetTempFile('IL');
    TempName := Copy(TempName, 0, Length(TempName) - 3) + Ext;
    try
      FStr := TFileStream.Create(TempName, fmCreate);
      Stream.Position := 0;
      FStr.CopyFrom(Stream, 0); { Copy entire stream is '0'}
    finally
      if FStr <> nil then FStr.Free;
    end;

    { AceMemoU.GForm is a public form that I sometimes
      use for purposes such as this }
    Image := TPMultiImage.Create(acememou.GForm);
    Image.Parent := acememou.GForm;
    Image.ImageName := TempName;
    Image.Width := R.Right - R.Left + 1;
    Image.Height := R.Bottom - R.Top + 1;
    Image.Stretch := True;
    Image.PaintTo(Handle, R.Left, R.Top);

  finally
    if Image <> nil then
    begin
      Image.Parent := nil;
      Image.Free;
    end;
    { Delete temporary file }
    if FileExists(TempName) then DeleteFile(TempName);
    if Stream <> nil then Stream.Free;
  end;
end;

initialization
  AceAddRecordType(AceIL_ImageLibType, AcePrintImageLibImage);

end.
